/**
 * CeboMsr - C++ example - Counter.
 *
 * This example shows how to use a counter. Digital I/O #1 is wired
 * to the counter, so every time this I/O is pulsed, the counter will
 * react on this event.
 *
 * Requirements: IO-1 must be connected to CNT (IO-0 on Cebo Stick).
 *
 * (c) 2013 Cesys GmbH
 * @author Thomas Hoppe
 */

#include <cebomsrpp.h>

#include <stdexcept>
#include <iostream>

using namespace std;
using namespace CeboMsr;

// Configuration and running.
void counterTest(Device &device) {
	// Create local copies, this shortens the source.
	DigitalPort dp0 = device.getDigitalPorts().at(0);
	Counter cnt = device.getCounters().at(0);

	// Set IO-1 as output.
	dp0.setOutputEnableMask(0x02);

	// Enable the counter.
	cnt.setEnabled(true);

	// Check the counters current value.
	cout << "Counter before starting: " << cnt.read() << endl;

	// Pulse IO-1 3 times.
	for (int i = 0; i < 3; ++i) {
		dp0.write(0x02);
		dp0.write(0x00);
	}

	// Check the counters current value.
	cout << "Counter should be 3: " << cnt.read() << endl;

	// Reset and ...
	cnt.reset();

	// ... disable the counter.
	cnt.setEnabled(false);

	// Check the counters current value.
	cout << "Counter should be 0: " << cnt.read() << endl;

	// Pulse IO-1 3 times, the counter should ignore these pulses.
	for (int i = 0; i < 3; ++i) {
		dp0.write(0x02);
		dp0.write(0x00);
	}

	// Check the counters current value.
	cout << "Counter should still be 0: " << cnt.read() << endl;
}

// Application entry point.
int main() {
	try {
		// Search for devices ...
		DeviceVector devices = LibraryInterface::enumerate(DeviceType::All);

		// If at least one has been found, use the first one ...
		if (!devices.empty()) {
			Device device = devices[0];

			// Open device, nothing can be done without doing this.
			device.open();

			// *** DIGITAL PORTS AND ANALOG OUTPUTS ARE MODIFIED DURING THIS ***
			// *** SECTION, SO DO NOT CONNECT ANY PERIPHERALS WITHOUT STUDYING ***
			// *** THE ACTIONS DONE HERE, OTHERWISE YOUR PERIPHERALS MAY BE DAMAGED !!! ***
			counterTest(device);

			// Finalize device usage, this free's up the device, so it can be used
			// again, including other applications.
			device.close();
		}
	}
	catch (const std::exception &e) { cout << e.what() << endl; }

	return 0;
}
