﻿using System;
using System.Collections.Generic;
using CeboMsrNet;
using System.IO;

/**
 * CeboMsr - C# example - Streaming Input.
 *
 * Multiple inputs are read multiple times at 300 Hz
 * and printed to the console.
 *
 * @author Thomas Hoppe
 */
namespace StreamingIExample
{
    class StreamingIExample
    {
        /// <summary>
        /// Read blocks of data from device.
        /// </summary>
        /// <param name="device">Device to use for test.</param>
        private void StreamingInputTestLC(Device device)
        {
            // Construct selection that contains inputs to read from.
            IInput[] inputs = new IInput[] {
                device.SingleEndedInputs[0],
                device.SingleEndedInputs[1],
                device.DifferentialInputs[1],
                device.DigitalPorts[0],
                device.DigitalPorts[1]
            };

            // Prepare device with this collection ...
            device.SetupInputFrame(inputs);

            // Start sampling ...
            device.StartBufferedDataAcquisition(300, 20 * 25, false);

            // Read 20 x 25 frames using blocked read,
            // this function returns after *all* (25) requested frames are collected.
            for (int i = 0; i < 20; ++i)
            {
                // Read 25 frames ...
                IList<InputFrame> frames = device.ReadBlocking(25);

                // Write out the 1st one.
                InputFrame inFrame = frames[0];
                Console.WriteLine(
                    "DigitalPort #0: " + inFrame.GetDigitalPort(0) + ", " +
                    "DigitalPort #1: " + inFrame.GetDigitalPort(1) + ", " +
                    "SingleEnded #0: " + inFrame.GetSingleEnded(0) + " V, " +
                    "SingleEnded #1: " + inFrame.GetSingleEnded(1) + " V, " +
                    "Differential #1: " + inFrame.GetDifferential(1) + " V");
            }

            // Stop the DAQ.
            device.StopDataAcquisition();
        }

        /// <summary>
        /// Read blocks of data from device.
        /// </summary>
        /// <param name="device">Device to use for test.</param>
        private void StreamingInputTestStick(Device device)
        {
            // Construct selection that contains inputs to read from.
            IInput[] inputs = new IInput[] {
                device.SingleEndedInputs[0],
                device.SingleEndedInputs[1],
                device.DigitalPorts[0]
            };

            // Prepare device with this collection ...
            device.SetupInputFrame(inputs);

            // Start sampling ...
            device.StartBufferedDataAcquisition(300, 20 * 25, false);

            // Read 20 x 25 frames using blocked read,
            // this function returns after *all* (25) requested frames are collected.
            for (int i = 0; i < 20; ++i)
            {
                // Read 25 frames ...
                IList<InputFrame> frames = device.ReadBlocking(25);

                // Write out the 1st one.
                InputFrame inFrame = frames[0];
                Console.WriteLine(
                    "DigitalPort #0: " + inFrame.GetDigitalPort(0) + ", " +
                    "SingleEnded #0: " + inFrame.GetSingleEnded(0) + " V, " +
                    "SingleEnded #1: " + inFrame.GetSingleEnded(1) + " V");
            }

            // Stop the DAQ.
            device.StopDataAcquisition();
        }

        /// <summary>
        /// The examples main method.
        /// </summary>
        private void RunExample()
        {
            try
            {
                // Search for devices ...
                IList<Device> devices = LibraryInterface.Enumerate(DeviceType.All);

                // If at least one has been found, use the first one ...
                if (devices.Count > 0)
                {
                    Device device = devices[0];

                    // Open device, nothing can be done without doing this.
                    device.Open();

                    // Do some streaming ...
                    if (device.DeviceType == DeviceType.CeboLC)
                        StreamingInputTestLC(device);
                    else if (device.DeviceType == DeviceType.CeboStick)
                        StreamingInputTestStick(device);
                    else
                        throw new InvalidOperationException("No supported device detected.");

                    // Finalize device usage, this free's up the device, so it can be used
                    // again, including other applications.
                    device.Close();
                }
            }
            catch (IOException ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        static void Main(string[] args)
        {
            new StreamingIExample().RunExample();
        }
    }
}
